package ACU::Log;

use v5.10.1;
use strict;
use warnings;
use Data::Dumper;
use Exporter 'import';
use POSIX qw(strftime);
use Term::ANSIColor qw(:constants);

use constant {
    FATAL => 1,
    ERROR2 => 2,
    ERROR => 3,
    WARN4 => 4,
    WARN => 5,
    USAGE => 6,
    INFO => 7,
    DEBUG => 8,
    TRACE => 9,
};

our @EXPORT    = qw(log FATAL ERROR2 ERROR WARN4 WARN USAGE INFO DEBUG TRACE);

our $display_level = 7;
our $save_level = 9;
our $fatal_error = 1;
our $fatal_warn = 0;

our $log_file = $0.".log";
my $log_fd;

sub log($@)
{
    my $level = shift;

    if (!$log_fd && $log_file) {
	open ($log_fd, ">>", $log_file) or die("Unable to open log ($log_file) file for writing");
	say $log_fd strftime("%a %b %e %H:%M:%S %Y", localtime), " START new logging session ";
    }

    if ($level <= $save_level and $log_fd) {
	print $log_fd strftime("%a %b %e %H:%M:%S %Y", localtime), " ", levelstr($level), " ";

	if ($level >= TRACE) {
	    print $log_fd Dumper(@_);
	}
	else {
	    say $log_fd @_;
	}
    }
    if ($level <= $display_level) {
	say STDERR (leveldisp($level), @_, RESET);
    }

    if ($fatal_warn && $level <= WARN){
	#TODO Thibaut
	#log(INFO, "Program stopped due to warning");
	exit 125;
    }
    elsif ($fatal_error && $level <= ERROR) {
	#TODO Thibaut
	#log(INFO, "Program stopped due to error");
	exit 126;
    }
    elsif ($level <= FATAL) {
	#TODO Thibaut
	#log(INFO, "Program stopped due to fatal error");
	exit 127;
    }
}

sub levelstr($)
{
    my $level = shift;

    return "FATAL" if ($level == 1);
    return "ERROR" if ($level == 3 or $level == 2);
    return "WARN " if ($level == 5 or $level == 4);
    return "USAGE" if ($level == 6);
    return "INFO " if ($level == 7);
    return "DEBUG" if ($level == 8);
    return "TRACE";
}

sub leveldisp($)
{
    my $level = shift;

    return BOLD, ON_RED, YELLOW, "/!\\", RESET, " ", BOLD if ($level == 1);
    return BOLD, ON_RED, ">>>", RESET, " ", BOLD if ($level == 2);
    return BOLD, RED, ">>>", RESET, " ", BOLD if ($level == 3);
    return BOLD, YELLOW, ">>>", RESET, " ", BOLD if ($level == 5 or $level == 4);
    return BOLD, MAGENTA, " * ", RESET, " ", BOLD if ($level == 6);
    return BOLD, CYAN, " * ", RESET, " " if ($level == 7);
    return BOLD, BLUE, " % ", RESET, " "  if ($level == 8);
    return BOLD, BLUE, "#", RESET, "   ";
}

1;
